@@ -2,7 +2,7 @@
 #
 # mpeg parser test harness
 #
-# $Id: mpegcat.c,v 1.4 2005/03/03 10:25:03 nemesis Exp $
+# $Id: mpegcat.c,v 1.7 2005/03/20 20:28:20 nemesis Exp $
 #
 # Copyright (C) 2001-2005 Kees Cook
 # kees@outflux.net, http://outflux.net/
@@ -936,6 +936,7 @@ int main(int argc, char *argv[])
     uint8_t byte;
     char *report_format;
 
+    /*
     optind = handle_options(argc, argv);
 
     if (optind >= argc)
@@ -1004,11 +1005,14 @@ int main(int argc, char *argv[])
         }
         printf("\n");
     }
+    */
+    optind=1;
 
     for (i = optind; i < argc; i++)
     {
         mpegparse * parser;
         off_t location=0;
+        off_t next_offset=0;
         
         if (!(parser=mpegparse_open_program_stream(argv[i]))) {
             perror(argv[i]);
@@ -1049,12 +1053,7 @@ int main(int argc, char *argv[])
                     report = "System";
                     break;
                 case MPEGPARSE_PES_VIDEO:
-                    if (info->header.pes_video.PES_scrambling_control) {
-                        report = "Video (scrambled)";
-                    }
-                    else {
-                        report = "Video";
-                    }
+                    report = "Video";
                     break;
                 case MPEGPARSE_VIDEO_GOP:
                     report = "GOP";
@@ -1072,12 +1071,7 @@ int main(int argc, char *argv[])
                     report = "Video Slice";
                     break;
                 case MPEGPARSE_PES_AUDIO:
-                    if (info->header.pes_audio.PES_scrambling_control) {
-                        report = "Audio (scrambled)";
-                    }
-                    else {
-                        report = "Audio";
-                    }
+                    report = "Audio";
                     break;
                 case MPEGPARSE_PES_PS_MAP:
                     report = "Program Stream Map";
@@ -1111,13 +1105,17 @@ int main(int argc, char *argv[])
                     break;
             }
             if (report) {
-                printf("%11llu: 0x%02X: (header %u data %u) %s\n",
-                    info->code_offset, info->code,
-                    info->header_length,
-                    info->data_length, report);
+                printf("%11llu (%s): 0x%02X: (header %u data %u) %s%s\n",
+                    info->code_offset,
+                    info->code_offset == next_offset ? "okay" : "SKIP",
+                    info->code,
+                    info->header_length + 4, // add the start code
+                    info->data_length, report,
+                    info->header_parsed ? "" : " NEEDS PARSER");
+                next_offset = info->code_offset + 4 + info->header_length + info->data_length;
             }
 
-            if (info->packet<=0) break;
+            if (info->packet<=MPEGPARSE_END) break;
         }
         mpegparse_shutdown(parser);
     }
