@@ -2,7 +2,7 @@
 #
 # mpeg parser library
 #
-# $Id: mpegparse.c,v 1.3 2005/03/03 10:25:03 nemesis Exp $
+# $Id: mpegparse.c,v 1.5 2005/03/20 20:18:33 nemesis Exp $
 #
 # Copyright (C) 2001-2005 Kees Cook
 # kees@outflux.net, http://outflux.net/
@@ -303,6 +303,9 @@ typedef struct
     // what kind of PES is it?
     packet_type packet;
 
+    // static size (valid for "PACK_SPECIAL")
+    size_t fixed_length;
+
     // how do we find the stream sub id
     subid_type subid_method;
 
@@ -312,52 +315,52 @@ packet_tag_info;
 
 packet_tag_info packet_tags[] = {
     // pic start
-    {0x00, 0x00, PACK_SPECIAL,     SUBID_NONE, MPEGPARSE_VIDEO_PICTURE, },
+    {0x00, 0x00, PACK_SPECIAL,     0, SUBID_NONE, MPEGPARSE_VIDEO_PICTURE, },
     // video slices
-    {0x01, 0xAF, PACK_SPECIAL,     SUBID_NONE, MPEGPARSE_VIDEO_SLICE, },
+    {0x01, 0xAF, PACK_SPECIAL,     0, SUBID_NONE, MPEGPARSE_VIDEO_SLICE, },
     // reserved
-    {0xB0, 0xB1, PACK_SPECIAL,     SUBID_NONE, MPEGPARSE_RESERVED, },
+    {0xB0, 0xB1, PACK_SPECIAL,     0, SUBID_NONE, MPEGPARSE_RESERVED, },
     // user data, sequences, ext
-    {0xB2, 0xB2, PACK_SPECIAL,     SUBID_NONE, MPEGPARSE_USER_DATA, },
-    {0xB3, 0xB3, PACK_SPECIAL,     SUBID_NONE, MPEGPARSE_SEQ_HEADER, },
-    {0xB4, 0xB4, PACK_SPECIAL,     SUBID_NONE, MPEGPARSE_SEQ_ERROR, },
-    {0xB5, 0xB5, PACK_SPECIAL,     SUBID_NONE, MPEGPARSE_EXTENSION_START, },
+    {0xB2, 0xB2, PACK_SPECIAL,     0, SUBID_NONE, MPEGPARSE_USER_DATA, },
+    {0xB3, 0xB3, PACK_SPECIAL,     0, SUBID_NONE, MPEGPARSE_SEQ_HEADER, },
+    {0xB4, 0xB4, PACK_SPECIAL,     0, SUBID_NONE, MPEGPARSE_SEQ_ERROR, },
+    {0xB5, 0xB5, PACK_SPECIAL,     0, SUBID_NONE, MPEGPARSE_EXTENSION_START, },
     // reserved
-    {0xB6, 0xB6, PACK_SPECIAL,     SUBID_NONE, MPEGPARSE_RESERVED, },
+    {0xB6, 0xB6, PACK_SPECIAL,     0, SUBID_NONE, MPEGPARSE_RESERVED, },
     // sequence, gop, ps end
-    {0xB7, 0xB7, PACK_SPECIAL,     SUBID_NONE, MPEGPARSE_VIDEO_SEQ_START, },
-    {0xB8, 0xB8, PACK_SPECIAL,     SUBID_NONE, MPEGPARSE_VIDEO_GOP, },
-    {0xB9, 0xB9, PACK_SPECIAL,     SUBID_NONE, MPEGPARSE_PROGRAM_END, },
+    {0xB7, 0xB7, PACK_SPECIAL,     0, SUBID_NONE, MPEGPARSE_VIDEO_SEQ_START, },
+    {0xB8, 0xB8, PACK_SPECIAL,     0, SUBID_NONE, MPEGPARSE_VIDEO_GOP, },
+    {0xB9, 0xB9, PACK_SPECIAL,     0, SUBID_NONE, MPEGPARSE_PROGRAM_END, },
     // pack
-    {0xBA, 0xBA, PACK_SPECIAL,     SUBID_NONE, MPEGPARSE_PACK, },
+    {0xBA, 0xBA, PACK_SPECIAL,     10, SUBID_NONE, MPEGPARSE_PACK, },
     // system: same len as PES
-    {0xBB, 0xBB, PACK_PES_SIMPLE,  SUBID_NONE, MPEGPARSE_SYSTEM, },
+    {0xBB, 0xBB, PACK_PES_SIMPLE,  0, SUBID_NONE, MPEGPARSE_SYSTEM, },
     // PES: prog stream map     *
-    {0xBC, 0xBC, PACK_PES_SIMPLE,  SUBID_NONE, MPEGPARSE_PES_PS_MAP, },
+    {0xBC, 0xBC, PACK_PES_SIMPLE,  0, SUBID_NONE, MPEGPARSE_PES_PS_MAP, },
     // PES: priv 1
-    {0xBD, 0xBD, PACK_PES_COMPLEX, SUBID_DATA, MPEGPARSE_PES_PRIV_1, },
+    {0xBD, 0xBD, PACK_PES_COMPLEX, 0, SUBID_DATA, MPEGPARSE_PES_PRIV_1, },
     // PES: padding, priv 2     *
-    {0xBE, 0xBE, PACK_PES_SIMPLE,  SUBID_NONE, MPEGPARSE_PES_PADDING, },
-    {0xBF, 0xBF, PACK_PES_SIMPLE,  SUBID_NONE, MPEGPARSE_PES_PRIV_2, },
+    {0xBE, 0xBE, PACK_PES_SIMPLE,  0, SUBID_NONE, MPEGPARSE_PES_PADDING, },
+    {0xBF, 0xBF, PACK_PES_SIMPLE,  0, SUBID_NONE, MPEGPARSE_PES_PRIV_2, },
     // PES: Audio
-    {0xC0, 0xDF, PACK_PES_COMPLEX, SUBID_NONE, MPEGPARSE_PES_AUDIO, },
+    {0xC0, 0xDF, PACK_PES_COMPLEX, 0, SUBID_NONE, MPEGPARSE_PES_AUDIO, },
     // PES: Video
-    {0xE0, 0xEF, PACK_PES_COMPLEX, SUBID_NONE, MPEGPARSE_PES_VIDEO, },
+    {0xE0, 0xEF, PACK_PES_COMPLEX, 0, SUBID_NONE, MPEGPARSE_PES_VIDEO, },
     // PES: ecm, emm, dsmcc     *
-    {0xF0, 0xF2, PACK_PES_SIMPLE,  SUBID_NONE, MPEGPARSE_OTHER, },
+    {0xF0, 0xF2, PACK_PES_SIMPLE,  0, SUBID_NONE, MPEGPARSE_OTHER, },
     // PES: iso 13522/h2221a-d
-    {0xF3, 0xF7, PACK_PES_COMPLEX, SUBID_NONE, MPEGPARSE_PES_ISO, },
+    {0xF3, 0xF7, PACK_PES_COMPLEX, 0, SUBID_NONE, MPEGPARSE_PES_ISO, },
     // PES: h2221e              *
-    {0xF8, 0xF8, PACK_PES_SIMPLE,  SUBID_NONE, MPEGPARSE_OTHER, },
+    {0xF8, 0xF8, PACK_PES_SIMPLE,  0, SUBID_NONE, MPEGPARSE_OTHER, },
     // PES: ancillary
-    {0xF9, 0xF9, PACK_PES_COMPLEX, SUBID_NONE, MPEGPARSE_PES_ANCILLARY, },
+    {0xF9, 0xF9, PACK_PES_COMPLEX, 0, SUBID_NONE, MPEGPARSE_PES_ANCILLARY, },
     // PES: reserved
-    {0xFA, 0xFE, PACK_PES_SIMPLE,  SUBID_NONE, MPEGPARSE_RESERVED, },
+    {0xFA, 0xFE, PACK_PES_SIMPLE,  0, SUBID_NONE, MPEGPARSE_RESERVED, },
     // PES: prog stream dir     *
-    {0xFF, 0xFF, PACK_PES_SIMPLE,  SUBID_NONE, MPEGPARSE_PES_PS_DIRECTORY, },
+    {0xFF, 0xFF, PACK_PES_SIMPLE,  0, SUBID_NONE, MPEGPARSE_PES_PS_DIRECTORY, },
 
     // end of list
-    {0, 0, PACK_NONE, SUBID_NONE, MPEGPARSE_END, }
+    {0, 0, PACK_NONE, 0, SUBID_NONE, MPEGPARSE_END, }
 };
 
 int mpegparse_handle_options(int argc, char *argv[])
@@ -503,6 +506,77 @@ int mpegparse_handle_options(int argc, char *argv[])
     return optind;
 }
 
+// load the specific bit fields from a given packet
+void
+mpegparse_load_pack_special(file_buf * fb, struct mpegparse_packet_info * info)
+{
+#define MPEGPARSE_HEADER_LEN_MAX 10
+    uint8_t bytes[10];
+
+    if (info->header_length > MPEGPARSE_HEADER_LEN_MAX) {
+        fprintf(stderr,"Oops: packet 0x%02X has header size %d but the parser\n",               info->code,info->header_length);
+        fprintf(stderr,"      only handles headers size %d or smaller!?\n",
+                MPEGPARSE_HEADER_LEN_MAX);
+        return;
+    }
+
+    if (buffer_look_ahead(fb, bytes, info->header_length) != FILE_BUF_OKAY) {
+        info->packet = MPEGPARSE_END;
+        return;
+    }
+
+    switch (info->code) {
+        case PES_TYPE_pack_start:
+            //|   0   |   1   |   2   |   3   |   4   |   5   |   6   |
+            // 76543210765432107654321076543210765432107654321076543210
+            // xxHHHxMMMMMMMMMMMMMMMxLLLLLLLLLLLLLLLxEEEEEEEEExRRRRRRRR
+            //   333 222222222211111 111110000000000 000000000 22111111// bit
+            //   210 987654321098765 432109876543210 876543210 10987654//     number
+            //     -  -       -    -  -       -    -  -      -        -
+            //     2  2       2    1  1       0    -  0      -        1// left
+            //     7  8       0    2  3       5    3  7      1        4//      shift
+            //
+            //|   7   |   8   |   9   |
+            // 765432107654321076543210
+            // RRRRRRRRRRRRRRxxrrrrrSSS
+            // 11110000000000  00000000   // bit
+            // 32109876543210  43210210   //     number
+            //        -     -      -  -
+            //        0     -      -  0   // left
+            //        6     2      3  0   //      shift
+
+            info->header.pack.system_clock_ref_base =
+                                        (((bytes[0] & 0x38) << 27) |
+                                         ((bytes[0] & 0x03) << 28) |
+                                         ((bytes[1] & 0xFF) << 20) |
+                                         ((bytes[2] & 0xF8) << 12) |
+                                         ((bytes[2] & 0x03) << 13) |
+                                         ((bytes[3] & 0xFF) << 5) |
+                                         ((bytes[4] & 0xF8) >> 3));
+            info->header.pack.system_clock_ref_ext =
+                                       (((bytes[4] & 0x03) << 7) |
+                                        ((bytes[5] & 0xFE) >> 1));
+            info->header.pack.program_mux_rate =
+                                   (((bytes[6] & 0xFF) << 14) |
+                                    ((bytes[7] & 0xFF) << 6) |
+                                    ((bytes[8] & 0xFC) >> 2));
+            info->header.pack.pack_stuffing_length = (bytes[9] & 0x07);
+
+            // FIXME: check marker bits
+
+            // adjust header length to include stuffing data.
+            // or should this be considered part of the "data" portion?
+            info->header_length+=info->header.pack.pack_stuffing_length;
+
+            info->header_parsed = 1;
+            break;
+
+        default:
+            break;
+    }
+}
+
+
 static char *length_format;
 
 uint8_t packet_buffer[65536];   // max size of a double byte
@@ -591,8 +665,9 @@ mpegparse_examine_code(mpegparse * parser, uint8_t code, off_t * position)
     parser->info.header_length = 0;
     parser->info.data_offset = 0;
     parser->info.data_length = 0;
-    parser->info.packet = MPEGPARSE_NONE;;
+    parser->info.packet = MPEGPARSE_NONE;
     parser->info.code = code;
+    parser->info.header_parsed = 0;
 
     // find the packet type
     for (i = 0; packet_tags[i].packet != PACK_NONE; i++)
@@ -604,12 +679,10 @@ mpegparse_examine_code(mpegparse * parser, uint8_t code, off_t * position)
             int sid = 0;
 
             parser->info.packet = packet_tags[i].parse_name;
+            parser->info.header_length = packet_tags[i].fixed_length;
 
             switch (packet_tags[i].packet)
             {
-                case PACK_SPECIAL:
-                    // actually, nothing special
-                    break;
                 case PACK_PES_SIMPLE:
                 case PACK_PES_COMPLEX:
                     if (packet_tags[i].packet == PACK_PES_COMPLEX)
@@ -709,6 +782,12 @@ mpegparse_examine_code(mpegparse * parser, uint8_t code, off_t * position)
                         *position = buffer_tell(fb) + length + 2;
                     }
 
+                    parser->info.header_parsed = 1;
+
+                    break;
+                case PACK_SPECIAL:
+                    // get additional details
+                    mpegparse_load_pack_special(fb,&parser->info);
                     break;
                 default:
                 case PACK_NONE:
@@ -1224,6 +1303,38 @@ mpegparse_find_packet(mpegparse * parser, off_t * location)
     return NULL;
 }
 
+// write the packet contents to a user buffer (that has already
+// been allocated to size 4 + info->header_length + info->data_length
+int mpegparse_get_raw_packet_contents(mpegparse * parser, uint8_t * bytes)
+{
+    int result=0;
+
+    size_t length = parser->info.header_length + parser->info.data_length;
+
+    // rewind to the code start
+    //buffer_seek(parser->fb,parser->info.code_offset);
+
+    // assemble code start
+    bytes[0]=0;
+    bytes[1]=0;
+    bytes[2]=1;
+    bytes[3]=parser->info.code;
+
+    if (buffer_look_ahead(parser->fb,bytes+4, length) == FILE_BUF_OKAY)
+        result=1;
+
+    // skip back past the code
+    //buffer_seek(parser->fb,parser->info.code_offset+4);
+
+    return result;
+}
+
+// report position in file
+off_t mpegparse_tell(mpegparse * parser)
+{
+    return buffer_tell(parser->fb);
+}
+
 void mpegparse_shutdown(mpegparse * parser)
 {
     if (!parser) return;
